@@ -29,6 +29,26 @@ import net.morilib.range.Range;
  */
 public interface NinaAction {
 
+	/**
+	 * 
+	 */
+	public static final int NO_CAPTURE = 1;
+
+	/**
+	 * 
+	 */
+	public static final int REGEX_ATOMIC = 2;
+
+	/**
+	 * 
+	 */
+	public static final int REGEX_LOOKAHEAD = 3;
+
+	/**
+	 * 
+	 */
+	public static final int REGEX_LOOKAHEAD_NEGATIVE = 4;
+
 	/**
 	 * 
 	 * @param e
@@ -75,11 +95,14 @@ public interface NinaAction {
 	public void setEdge(NinaEvent q, Object o);
 
 	/**
+	 * 
 	 * @param q
 	 * @param o
+	 * @param qc
+	 * @param qf
 	 */
 	public void setEdgeCharSequence(NinaEvent q, CharSequence o,
-			int qc);
+			int qc, int qf);
 
 	/**
 	 * 
@@ -122,11 +145,12 @@ public interface NinaAction {
 	/**
 	 * 
 	 * @param q
-	 * @param p
 	 * @param s
+	 * @param flag
 	 */
-	public void setEdgeNFA(NinaEvent q, NinaParser p,
-			NFA<Object, NFAState, Void> s);
+	public void setEdgeNFA(NinaEvent q,
+			NFA<Object, NFAState, Integer> s,
+			int flag);
 
 	/**
 	 * 
@@ -135,6 +159,13 @@ public interface NinaAction {
 	 */
 	public void setEdgeDynamic(NinaEvent q, String var);
 
+	/**
+	 * 
+	 * @param q
+	 * @param c
+	 */
+	public void setEdgeBackslash(NinaEvent q, int c);
+
 	/**
 	 * 
 	 * @param q
@@ -318,4 +349,36 @@ public interface NinaAction {
 	 */
 	public void setEdgeSkipSpaces(NinaEvent q, String str);
 
+	/**
+	 * 
+	 * @param q
+	 * @param l
+	 * @param cs
+	 */
+	public void setEdgeStringSequence(NinaEvent q, int l, String cs);
+
+	/**
+	 * 
+	 * @param q
+	 * @param l
+	 * @param flg
+	 */
+	public void setEdgeFormatInteger(NinaEvent q, int l, String flg);
+
+	/**
+	 * 
+	 * @param q
+	 * @param l
+	 * @param flg
+	 */
+	public void setEdgeFormatHex(NinaEvent q, int l, String flg);
+
+	/**
+	 * 
+	 * @param q
+	 * @param l
+	 * @param flg
+	 */
+	public void setEdgeFormatOct(NinaEvent q, int l, String flg);
+
 }
