@@ -15,10 +15,10 @@
  */
 package net.morilib.nina;
 
-import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.LineNumberReader;
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.Reader;
@@ -125,6 +125,7 @@ public class Quadro implements NinaEvent {
 	String name;
 	String valtype;
 	String description;
+	private int fragmentLineno;
 
 	//
 	private NinaLanguageOption langopt = null;
@@ -963,13 +964,14 @@ public class Quadro implements NinaEvent {
 		Map<String, String> m = new HashMap<String, String>();
 		List<int[]> l = new ArrayList<int[]>(), z = null;
 		List<String[]> d = new ArrayList<String[]>();
-		BufferedReader br = new BufferedReader(rd);
+		LineNumberReader br = new LineNumberReader(rd);
 		List<String> sl = new ArrayList<String>();
 		boolean b = false, b2 = true, nf = false;
 		String s, x = null, desc = "";
 		StringWriter sw = null;
 		PrintWriter pw = null;
 		int y = 0, p = 0;
+		int lineno;
 		LRObject lr;
 		String[] a;
 		Matcher h;
@@ -1044,6 +1046,7 @@ public class Quadro implements NinaEvent {
 
 		if(sw != null)  throw new NinaException("unexpectedeof");
 		z = setquadro(q, m, x, y, z, l);
+		lineno = br.getLineNumber();
 		sw = new StringWriter();
 		pw = new PrintWriter(sw, true);
 		while((s = br.readLine()) != null)  pw.println(s);
@@ -1052,6 +1055,7 @@ public class Quadro implements NinaEvent {
 		v.lint.setYOffset(0);
 		v.ids = d;
 		v.description = desc;
+		v.fragmentLineno = lineno + 1;
 		iswarning = v.lint.validate(errst) | iswarning;
 		return v;
 	}
@@ -1178,6 +1182,14 @@ public class Quadro implements NinaEvent {
 		return fragment != null ? fragment : "";
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	public int getFragmentLineno() {
+		return fragmentLineno;
+	}
+
 	/**
 	 * 
 	 * @return
