@@ -49,6 +49,7 @@ import net.morilib.nina.cmd.NinatOptions;
 import net.morilib.nina.metalang.NinaMetaLanguage;
 import net.morilib.nina.translate.sh.ShNinatBuiltInCommands;
 import net.morilib.nina.translate.sh.ShNinatFileSystem;
+import net.morilib.range.CharSets;
 import net.morilib.range.Interval;
 import net.morilib.range.Range;
 import net.morilib.sh.DefaultShRuntime;
@@ -71,6 +72,12 @@ import net.morilib.util.Tuple2;
 public abstract class AbstractNinaTranslator
 implements NinaTranslator {
 
+	//
+	static final Range JAVA_IDENTIFIER_START =
+			CharSets.parse("\\p{JavaIdentifierStart}");
+	static final Range JAVA_IDENTIFIER_PART =
+			CharSets.parse("\\p{JavaIdentifierPart}");
+
 	//
 	private static final SimpleDateFormat YEAR =
 			new SimpleDateFormat("yyyy");
@@ -102,6 +109,7 @@ implements NinaTranslator {
 
 	//
 	private int stateNo = 0;
+	private int rangeNo = 0;
 	private Map<DFAState<Object, ?, Void>, Integer> states =
 			new IdentityHashMap
 			<DFAState<Object, ?, Void>, Integer>();
@@ -110,6 +118,8 @@ implements NinaTranslator {
 	private Set<Integer> emptyTransitions = new HashSet<Integer>();
 	private Stack<DFAState<Object, ?, Void>> trz =
 			new Stack<DFAState<Object, ?, Void>>();
+	private Map<Range, String> rangeNameMap =
+			new HashMap<Range, String>();
 
 	//
 	private Map<String, AbstractNinaTranslator> subfas;
@@ -179,6 +189,36 @@ implements NinaTranslator {
 		}
 	}
 
+	/**
+	 * 
+	 * @param range
+	 * @return
+	 */
+	protected String getRangeName(Range range) {
+		AbstractNinaTranslator root;
+		String name;
+
+		for(root = this; root.parent != root; root = root.parent) {}
+		if(root.rangeNameMap.containsKey(range)) {
+			return root.rangeNameMap.get(range);
+		} else {
+			name = "isRange" + (root.rangeNo++);
+			root.rangeNameMap.put(range, name);
+			return name;
+		}
+	}
+
+	/**
+	 * 
+	 * @return
+	 */
+	protected Set<Range> getRangeSet() {
+		AbstractNinaTranslator root;
+
+		for(root = this; root.parent != root; root = root.parent) {}
+		return root.rangeNameMap.keySet();
+	}
+
 	/**
 	 * 
 	 * @return
@@ -437,6 +477,12 @@ implements NinaTranslator {
 	 */
 	protected abstract void printConstants(PrintStream out);
 
+	/**
+	 * 
+	 * @param out
+	 */
+	protected abstract void printRangeMethods(PrintStream out);
+
 	/**
 	 * 
 	 * @param ot
@@ -636,7 +682,7 @@ implements NinaTranslator {
 			NinaSubautomata sub) {
 		ShEnvironment env;
 		StringBuffer b;
-		String s, d;
+		String s, d, tmp;
 
 		env  = new ShRootEnvironment();
 		env.bind("IFS", " \t");
@@ -685,6 +731,19 @@ implements NinaTranslator {
 		env.bind("COMMENT", q.getOptionNvl("comment", ""));
 		env.bind("COMMENTHEADER", escapeHeader(q.getOptionNvl(
 				"commentHeader", "#")));
+		env.bind("CUSTOMREADER", q.isOptionDefined("customReader") ?
+				"true" : "");
+		tmp = q.getOptionNvl("lineContinuation", "");
+		if(tmp.equals("")) {
+			// do nothing
+		} else if(tmp.charAt(0) == '\\') {
+			tmp = tmp.substring(0, 2);
+		} else {
+			tmp = tmp.substring(0, 1);
+		}
+		env.bind("LINECONT", tmp);
+		env.bind("USE_IMMEDIATE_FN",
+				q.isOptionDefined("useImmediateFunction") ? "t" : "");
 
 		if("prompt".equals(q.getOption("main"))) {
 			env.bind("PUTPROMPT", "true");
@@ -786,7 +845,7 @@ implements NinaTranslator {
 	 */
 	@SuppressWarnings("unchecked")
 	protected void translateQuadro(String name, Object o,
-			NinaInfo opts, Map<String, String> prms,
+			final NinaInfo opts, Map<String, String> prms,
 			final NinaSubautomata sub,
 			PrintStream ot) throws IOException {
 		ShNinatBuiltInCommands cmd;
@@ -1035,6 +1094,34 @@ implements NinaTranslator {
 				return 0;
 			}
 
+		});
+		cmd.putCommand("print_range_methods", new ShProcess() {
+
+			@Override
+			public int main(ShEnvironment env, ShFileSystem fs,
+					InputStream stdin, PrintStream stdout,
+					PrintStream stderr,
+					String... args) throws IOException {
+				printRangeMethods(stdout);
+				return 0;
+			}
+
+		});
+		cmd.putCommand("print_line_directive", new ShProcess() {
+
+			@Override
+			public int main(ShEnvironment env, ShFileSystem fs,
+					InputStream stdin, PrintStream stdout,
+					PrintStream stderr,
+					String... args) throws IOException {
+				if(!quadro.isOptionDefined("suppressLineDirective")) {
+					stdout.printf("#line %d \"%s\"\n",
+							quadro.getFragmentLineno(),
+							opts.getOriginalFilename());
+				}
+				return 0;
+			}
+
 		});
 
 		if(o instanceof NFABuilder) {
@@ -1070,7 +1157,7 @@ implements NinaTranslator {
 	 * @see net.morilib.nina.compiler.NinaTranslater#translate(java.io.Reader, java.io.PrintWriter)
 	 */
 	@Override
-	public void translate(String name, Reader rd, NinaInfo opts,
+	public final void translate(String name, Reader rd, NinaInfo opts,
 			List<String> libs, Map<String, String> prms,
 			PrintStream ot) throws IOException {
 		NinaSubautomata s;
