@@ -32,6 +32,7 @@ import net.morilib.nina.Quadro;
 import net.morilib.nina.TMBuilder;
 import net.morilib.nina.translate.sh.ReplaceStrangeChar;
 import net.morilib.range.Interval;
+import net.morilib.range.Range;
 import net.morilib.sh.ShEnvironment;
 
 /**
@@ -67,6 +68,14 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		return o == Interval.FROM_INFIMUM ? 0 : ((Integer)o).intValue();
 	}
 
+	//
+	private int infimumbound(Range a) {
+		Interval v;
+
+		v = a.intervals().first();
+		return infimumbound(v);
+	}
+
 	//
 	private String tostr(int c) {
 		String x;
@@ -79,6 +88,8 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 			x = "'\\''";
 		} else if(Character.isISOControl(c)) {
 			x = Integer.toString(c);
+		} else if(c >= 0x80) {
+			x = Integer.toString(c);
 		} else if(c < Character.MAX_VALUE) {
 			x = String.format("'%c'", (char)c);
 		} else {
@@ -93,28 +104,66 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 	}
 
 	//
-	private void printintv(PrintStream out, Interval v, boolean els,
-			Object o) {
-		String s, t, a, x, y, z;
+	private void printintv1(PrintStream out, Interval v) {
+		String s, t, x, y;
 		int c, d;
 
 		s = v.isInfimumClosed() ? ">=" : ">";
 		t = v.isSupremumClosed() ? "<=" : "<";
-		a = els ? "\t\t\t} else if" : "\t\t\tif";
 		c = ((Integer)v.getInfimumBound()).intValue();
 		d = ((Integer)v.getSupremumBound()).intValue();
 		x = tostr(c);
 		y = tostr(d);
-		z = o != null && o.toString().startsWith(LOK) ?
-				"__l__ && " : "";
 		if(v.isClosed() && c == d) {
-			out.format("%s(%s__c__ == %s) {\n", a, z, x);
+			out.format("(__c__ == %s)", x);
 		} else {
-			out.format("%s(%s__c__ %s %s && __c__ %s %s) {\n",
-					a, z, s, x, t, y);
+			out.format("(__c__ %s %s && __c__ %s %s)", s, x, t, y);
 		}
 	}
 
+	//
+	private void printRange(PrintStream out, Range r) {
+		String d = "";
+
+		for(Interval v : r.intervals()) {
+			out.print(d);
+			printintv1(out, v);
+			d = " ||\n\t\t\t\t\t";
+		}
+	}
+
+	//
+	private void printRangeMethod(PrintStream out, Range r) {
+		String name;
+
+		name = getRangeName(r);
+		out.format("\n");
+		out.format("\tprivate bool %s(bool l, int __c__)\n\t{\n", name);
+		out.format("\t\treturn (l && (");
+		printRange(out, r);
+		out.format("));\n");
+		out.format("\t}\n");
+	}
+
+	//
+	private void printintv(PrintStream out, Range r, boolean els,
+			Object o) {
+		String a, name;
+		boolean lahead;
+
+		a = els ? "\t\t\t} else if" : "\t\t\tif";
+		out.format("%s(", a);
+		lahead = o != null && o.toString().startsWith(LOK);
+		if(getMachine().equals("dfa") && !lahead) {
+			name = getRangeName(r);
+			out.format("__this__.%s(true, __c__)", name);
+		} else {
+			name = getRangeName(r);
+			out.format("__this__.%s(__l__, __c__)", name);
+		}
+		out.format(") {\n");
+	}
+
 	//
 	private void printobj(PrintStream out, Object o, boolean els,
 			Object m) {
@@ -389,8 +438,9 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 	//
 	private void printState(PrintStream out,
 			DFAState<Object, ?, Void> dfa) {
-		boolean els = false;
 		int sn = getStateNo(dfa), c;
+		Map<Object, Range> z;
+		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
@@ -404,8 +454,11 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 			return;
 		}
 
+		// classify edges
+		z = classifyEdges(dfa);
+
 		// with lookahead
-		for(Interval v : dfa.getAlphabetRanges()) {
+		for(Range v : z.values()) {
 			if(v.isEmpty())  continue;
 			c = infimumbound(v);
 			if(v.isInfimumOpen())  c++;
@@ -418,7 +471,7 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		}
 
 		// without lookahead
-		for(Interval v : dfa.getAlphabetRanges()) {
+		for(Range v : z.values()) {
 			if(v.isEmpty())  continue;
 			c = infimumbound(v);
 			if(v.isInfimumOpen())  c++;
@@ -881,6 +934,16 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printRangeMethods(java.io.PrintStream)
+	 */
+	@Override
+	protected void printRangeMethods(PrintStream out) {
+		for(Range range : getRangeSet()) {
+			printRangeMethod(out, range);
+		}
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#reportStatistics(java.io.PrintStream)
 	 */
