@@ -105,9 +105,8 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	}
 
 	//
-	private void printintv1(PrintStream out, Interval v, boolean els,
-			Object o) {
-		String s, t, x, y, z;
+	private void printintv1(PrintStream out, Interval v) {
+		String s, t, x, y;
 		int c, d;
 
 		s = v.isInfimumClosed() ? ">=" : ">";
@@ -116,25 +115,52 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		d = ((Integer)v.getSupremumBound()).intValue();
 		x = tostr(c);
 		y = tostr(d);
-		z = isLookahead(o) ? "__l__ && " : "";
 		if(v.isClosed() && c == d) {
-			out.format("(%s$c == %s)", z, x);
+			out.format("($c == %s)", x);
 		} else {
-			out.format("(%s$c %s %s && $c %s %s)", z, s, x, t, y);
+			out.format("($c %s %s && $c %s %s)", s, x, t, y);
 		}
 	}
 
+	//
+	private void printRange(PrintStream out, Range r) {
+		String d = "";
+
+		for(Interval v : r.intervals()) {
+			out.print(d);
+			printintv1(out, v);
+			d = " ||\n\t\t\t\t\t";
+		}
+	}
+
+	//
+	private void printRangeMethod(PrintStream out, Range r) {
+		String name;
+
+		name = getRangeName(r);
+		out.format("\n");
+		out.format("\tprivate boolean %s(boolean l, int $c) {\n", name);
+		out.format("\t\treturn (l && (");
+		printRange(out, r);
+		out.format("));\n");
+		out.format("\t}\n");
+	}
+
 	//
 	private void printintv(PrintStream out, Range r, boolean els,
-			Object o) {
-		String a, d = "";
+			Object obj) {
+		String a, name;
+		boolean lahead;
 
 		a = els ? "\t\t\t} else if" : "\t\t\tif";
 		out.format("%s(", a);
-		for(Interval v : r.intervals()) {
-			out.print(d);
-			printintv1(out, v, els, o);
-			d = " || ";
+		lahead = isLookahead(obj);
+		if(getMachine().equals("dfa") && !lahead) {
+			name = getRangeName(r);
+			out.format("%s(true, $c)", name);
+		} else {
+			name = getRangeName(r);
+			out.format("%s(__l__, $c)", name);
 		}
 		out.format(") {\n");
 	}
@@ -965,6 +991,16 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printRangeMethods(java.io.PrintStream)
+	 */
+	@Override
+	protected void printRangeMethods(PrintStream out) {
+		for(Range range : getRangeSet()) {
+			printRangeMethod(out, range);
+		}
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#reportStatistics(java.io.PrintStream)
 	 */
