@@ -16,6 +16,7 @@
 #
 putlicense_cstyle.sh
 buffer='$buffer'
+bufferUnicode='$bufferUnicode'
 int='$int'
 num='$num'
 indent='$indent'
@@ -25,6 +26,7 @@ sq="'"
 echo '/**'
 awk '{ printf(" * %s\n", $0) }' description
 echo ' */'
+[ -n "$USE_IMMEDIATE_FN" ] && echo '(function() {'
 cat definition
 
 echo '/* @@@-PARSER-CODE-START-@@@ */'
@@ -50,9 +52,14 @@ cat << EOF
 	this.exception = null;
 
 	this.$buffer = null;
+	this.$bufferUnicode = null;
 	this.$int = null;
 	this.$num = null;
 
+	this.lineNumber = 1;
+
+	this.yieldIsEof = false;
+
 EOF
 
 [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
@@ -89,7 +96,9 @@ EOF
 if [ -n "$TYPE" ]; then
 elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
   echo '	this.__unreadcomment = -2;'
+  echo '	this.COMMENT = true;'
 fi
+[ -n "$COMMENT" ] && echo '	this.COMMENT = true;'
 
 for i in $SUBAUTOMATA
 do
@@ -97,6 +106,7 @@ do
   echo "	this.ENGINE_${ch1} = ${CLASSNAME}.ENGINE_${ch1};"
 done
 echo "	this._unreadl = null;"
+echo "	this._unreadc = null;"
 echo '}'
 echo
 echo "function ${CLASSNAME}_TokenException() {"
@@ -128,6 +138,41 @@ ${CLASSNAME}.__log = null;
 
 EOF
 
+function _putcom {
+echo
+echo "${CLASSNAME}.prototype._readComment = function(rd) {"
+if [ -n "$LINECONT" ]; then
+  cat << EOF
+	var c, c2;
+	if(this._unreadc !== null) {
+		c = this._unreadc;
+		this._unreadc = null;
+	} else {
+		c = this._readComment0(rd);
+	}
+	if(c === ${sq}${LINECONT}${sq}.charCodeAt(0)) {
+		c2 = this._readComment0(rd);
+		if(c2 === '\n'.charCodeAt(0)) {
+			return this._readComment(rd);
+		} else if(c2 === '\r'.charCodeAt(0)) {
+			this._unreadc = this._readComment0();
+			this._unreadc = this._unreadc === '\n'.charCodeAt(0) ? null : this._unreadc;
+			return this._readComment(rd);
+		} else {
+			this._unreadc = c2;
+			return c;
+		}
+	} else {
+		return c;
+	}
+EOF
+else
+  echo '	var c = this._readComment0(rd);'
+  echo "	return c;"
+fi
+echo '}'
+}
+
 function _putreadf {
   if [ -n "$ENABLE_INDENT" ]; then
     cat << EOF
@@ -161,6 +206,22 @@ ${CLASSNAME}.prototype.${1} = function(rd) {
 }
 EOF
   fi
+  cat << EOF
+${CLASSNAME}.prototype._readChar = function(rd) {
+	var c;
+
+	c = rd();
+	if(c === '\n'.charCodeAt(0)) {
+		this.lineNumber++;
+	}
+	return c;
+}
+EOF
+  [ -n "$CUSTOMREADER" ] || cat << EOF
+${CLASSNAME}.prototype._readFromStream = function(rd) {
+		return this._readChar(rd);
+}
+EOF
 }
 
 cat field
@@ -168,12 +229,12 @@ if [ -z "$LEXER" ]; then
   _putreadf _read1l
   if [ "$COMMENT" = "shell" ]; then
     cat << EOF
-${CLASSNAME}.prototype._readComment = function(rd) {
+${CLASSNAME}.prototype._readComment0 = function(rd) {
 	var c;
 
-	c = rd();
-	if(c === ${sq}${COMMENTHEADER}${sq}.charCodeAt(0)) {
-		while((c = rd()) !== null) {
+	c = this._readFromStream(rd);
+	if(this.COMMENT && c === ${sq}${COMMENTHEADER}${sq}.charCodeAt(0)) {
+		while((c = this._readFromStream(rd)) !== null) {
 			if(c === '\n'.charCodeAt(0)) {
 				break;
 			}
@@ -184,22 +245,25 @@ ${CLASSNAME}.prototype._readComment = function(rd) {
 	}
 };
 EOF
+  _putcom
   elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
     cat << EOF
-${CLASSNAME}.prototype._readComment = function(rd) {
+${CLASSNAME}.prototype._readComment0 = function(rd) {
 	var c, state = 0;
 
 	if(this.__unreadcomment === null || this.__unreadcomment >= 0) {
 		c = this.__unreadcomment;
 		this.__unreadcomment = -2;
 		return c;
-	} else if((c = rd()) == '/'.charCodeAt(0)) {
-		while((c = rd()) >= 0) {
+	} else if(!this.COMMENT) {
+		return this._readFromStream(rd);
+	} else if((c = this._readFromStream(rd)) == '/'.charCodeAt(0)) {
+		while((c = this._readFromStream(rd)) >= 0) {
 			switch(state) {
 			case 0:
-				if(c == '*'.charCodeAt(0)) {
+				if(c === '*'.charCodeAt(0)) {
 					state = 100;
-				} else if(c == '/'.charCodeAt(0)) {
+				} else if(c === '/'.charCodeAt(0)) {
 					state = 200;
 				} else {
 					this.__unreadcomment = c;
@@ -207,19 +271,19 @@ ${CLASSNAME}.prototype._readComment = function(rd) {
 				}
 				break;
 			case 100:
-				if(c == '*'.charCodeAt(0)) {
+				if(c === '*'.charCodeAt(0)) {
 					state = 101;
 				}
 				break;
 			case 101:
-				if(c == '/'.charCodeAt(0)) {
+				if(c === '/'.charCodeAt(0)) {
 					return ' '.charCodeAt(0);
-				} else {
+				} else  if(c !== '*'.charCodeAt(0)) {
 					state = 100;
 				}
 				break;
 			case 200:
-				if(c == '\n'.charCodeAt(0)) {
+				if(c === '\n'.charCodeAt(0)) {
 					return '\n'.charCodeAt(0);
 				}
 				break;
@@ -232,13 +296,16 @@ ${CLASSNAME}.prototype._readComment = function(rd) {
 	}
 };
 EOF
+  _putcom
   else
     cat << EOF
-${CLASSNAME}.prototype._readComment = function(rd) {
-	return rd();
+${CLASSNAME}.prototype._readComment0 = function(rd) {
+	return this._readFromStream(rd);
 };
 EOF
+  _putcom
   fi
+
   cat << EOF
 
 ${CLASSNAME}.prototype._read1 = function(stream) {
@@ -667,6 +734,8 @@ ${CLASSNAME}.ENGINE_${ch1}.toString = function() {
 EOF
 done
 
+print_range_methods
+
 for i in $LRPARSERS
 do
   y.tab.js.sh $i
@@ -835,10 +904,17 @@ EOF
 				this.__logprint("match failed: try backtracking: ", c);
 EOF
 cat << EOF
-			} else if(c == null) {
+			} else if(c === null) {
 EOF
 [ -n "$MATCHER" ] || cat << EOF
-				if(!b)  throw "${TOKENERROR}";
+				if(!b) {
+					if(this.yieldIsEof) {
+						st[0] = NINA_YIELD;
+						return null;
+					} else {
+						throw "${TOKENERROR}";
+					}
+				}
 EOF
 [ -n "$MATCHER" ] && cat << EOF
 				if(!b) {
@@ -875,7 +951,7 @@ EOF
 				this.__logprint("match failed: try backtracking: ", c);
 EOF
 cat << EOF
-			} else if(c == null) {
+			} else if(c === null) {
 				if(!b)  throw "${TOKENERROR}";
 				st[0] = NINA_ACCEPT;  return null;
 			} else {
@@ -923,7 +999,7 @@ ${CLASSNAME}.prototype.parse_generic = function(rd, entry) {
 
 	this.__logopen();
 	try {
-		if(this.__slen == 0) {
+		if(this.__slen === 0) {
 			b = true;
 			this.__stkpush(0, entry);
 		}
@@ -932,7 +1008,7 @@ ${CLASSNAME}.prototype.parse_generic = function(rd, entry) {
 			try {
 				if((c = this._parse(rd, c, b, a)) != null) {
 					// do nothing
-				} else if(a[0] == NINA_FAIL) {
+				} else if(a[0] === NINA_FAIL) {
 					while((this.__state = this.getdeadstate()) < 0) {
 						if((b = this.execfinally()) != null)  break ot;
 						if(this.__slen-- <= 1) {
@@ -943,15 +1019,15 @@ EOF
 						}
 					}
 					c = ${CLASSNAME}.__SKIP__;
-				} else if(a[0] == NINA_HALT_ACCEPT) {
+				} else if(a[0] === NINA_HALT_ACCEPT) {
 					if((b = this.execfinally()) != null)  break;
 					this.__slen = 0;
 					b = true;  break;
-				} else if(a[0] == NINA_HALT_REJECT) {
+				} else if(a[0] === NINA_HALT_REJECT) {
 					if((b = this.execfinally()) != null)  break;
 					this.__slen = 0;
 					b = false;  break;
-				} else if(a[0] == NINA_YIELD) {
+				} else if(a[0] === NINA_YIELD) {
 					return false;
 				} else if(this.__slen > 1) {
 					if((b = this.execfinally()) != null)  break;
@@ -981,10 +1057,25 @@ EOF
 	}
 }
 
-${CLASSNAME}.prototype.parse = function(rd) {
+${CLASSNAME}.prototype.parse = function(arg) {
+	var rd;
+
+	if(typeof arg === 'string') {
+		rd = ${CLASSNAME}.readFromString(arg);
+	} else {
+		rd = arg;
+	}
 	return this.parse_generic(rd, this.ENGINE_${ch1});
 }
 
+${CLASSNAME}.readFromString = function(s) {
+	var p = 0;
+
+	return function() {
+		return p < s.length ? s.charCodeAt(p++) : null;
+	};
+}
+
 ${CLASSNAME}.prototype.setStream = function(rd) {
 	if(this.befstream != null) {
 		throw "IllegalStateException";
@@ -1034,10 +1125,6 @@ ${CLASSNAME}.parseStdin = function() {
 	return ${CLASSNAME}.parseStream(
 			new java.io.InputStreamReader(java.lang.System['in']));
 }
-
-function puts(s) {
-	CONSOLE.println(s);
-}
 EOF
 
 for i in $SUBAUTOMATA
@@ -1050,12 +1137,20 @@ cat fragment
 #cat fragment | replace_action
 
 [ -n "$PUTMAIN" ] && cat << EOF
+function puts(s) {
+	CONSOLE.println(s);
+}
 ${CLASSNAME}.parseStdin();
 EOF
 [ -n "$PUTPROMPT" ] && cat << EOF
 var s;
 
+function puts(s) {
+	CONSOLE.println(s);
+}
+
 while((s = java.lang.System.console().readLine("> ")) != null) {
 	${CLASSNAME}.parseStream(new java.io.StringReader(s));
 }
 EOF
+[ -n "$USE_IMMEDIATE_FN" ] && echo '})();'
